-- this func distinguises between online multiplayer and offline single player
function is_online()
  if server_config==nil then return false; end -- for maped
  return not server_config:is_true("single_player")
end
function is_offline()
  return not is_online()
end

function is_maped()
  return type(hurt)=="nil" -- checks to see if hurt function defined, which is true for euoserv, false for maped
end

function is_player(id)

  return (id<100)
end

function trim(s)
  if s==nil then return nil; end
  -- from PiL2 20.4
  return (string.gsub(s,"^%s*(.-)%s*$", "%1"))
end

function call(fname, ...)

  local f=_G[fname]
  if(type(f)=="function") then return f(arg) end
end

function strip_non_alpha_chars(str)

  if str~=nil then

    str = string.gsub (str, "([^%w])","")
  end
  return str
end

function toggle_switch(id,x,y,z)

  local map=get_map_ptr(z)
  local c=map:get_item_code(x,y)

  if(c==Item(BCODE_SWITCH_ON)) then
    c=Item(BCODE_SWITCH_OFF)
  else
    c=Item(BCODE_SWITCH_ON)
  end
  map:set_item_code(x,y,c)
  update_fov(id)
end

function in_array(needle, haystack)

  for i,v in pairs(haystack) do
    if needle==v then return true end
  end
  return false
end

function array_key_exists(needle, haystack)

  for i,v in pairs(haystack) do
    if needle==i then return true end
  end
  return false
end

function find_key(needle, haystack)
  for i,v in pairs(haystack) do
    if needle==v then return i end
  end
  return false
end

function get_random_element(a)

  if #a==0 then return false; end
  return a[math.random(1,#a)]
end

-- scale a monster's hp, mr, as, dmg from lvl 1 to 100
-- formulas are roughly based on player vals (except hp) -
function set_monster_level(m,lvl)

  if lvl<1 then lvl=1 end
  if m==nil then return end

  local hp=lvl*4 -- down from *6
  local def=lvl/2+5
  local thaco=def+7
  local dd=lvl/18+2
  local ds=lvl/10+2
  local int=lvl*8
  local mr=lvl*3
  if mr>9999 then mr=9999; end

  m.lvl=lvl
  m:set_hp_max(hp)
  m.hp=hp
  m.mana=hp
  --m.xp_kill=math.pow(hp,1.15)/2 -- a bit over hp
  m.xp_kill=hp/2

  m:set_thaco(thaco)
  m:set_dmg_dice(dd)
  m:set_dmg_sides(ds)
  m:set_magic_resistance(mr)
  m:set_intel(int)
  m:set_base_ac(def)

  return m
end

function mkEvil(m)

  m:set_align(ALIGN_EVIL)
end

function mkGood(m)

  m:set_align(ALIGN_GOOD)
end

function set_neutral_aligned(m)

  m:set_align(ALIGN_NEUTRAL)

end

function dist(x,y,a,b)

  local dx=a-x
  local dy=b-y
  return math.sqrt(dx*dx+dy*dy)
end

-- explode(seperator, string)
-- from http://lua-users.org/wiki/SplitJoin
function explode(d,p)
  local t, ll, l
  t={}
  ll=0
  if(#p == 1) then return {p} end
  while true do
    l=string.find(p,d,ll,true) -- find the next d in the string
    if l~=nil then -- if "not not" found then..
      table.insert(t, string.sub(p,ll,l-1)) -- Save it in our array.
      ll=l+1 -- save just after where we found it for searching next time.
    else
      table.insert(t, string.sub(p,ll)) -- Save what's left in our array.
      break -- Break at end, as it should be, according to the lua manual.
    end
  end
  return t
end

-- Copyright 2009: hans@hpelbers.org
-- This is freeware
function print_r (t, name, indent)
  local tableList = {}
  function table_r (t, name, indent, full)
    local id = not full and name
    or type(name)~="number" and tostring(name) or '['..name..']'
    local tag = indent .. id .. ' = '
    local out = {} -- result
    if type(t) == "table" then
      if tableList[t] ~= nil then table.insert(out, tag .. '{} -- ' .. tableList[t] .. ' (self reference)')
      else
        tableList[t]= full and (full .. '.' .. id) or id
        if next(t) then -- Table not empty
          table.insert(out, tag .. '{')
          for key,value in pairs(t) do
            table.insert(out,table_r(value,key,indent .. '| ',tableList[t]))
          end
          table.insert(out,indent .. '}')
        else table.insert(out,tag .. '{}') end
      end
    else
      local val = type(t)~="number" and type(t)~="boolean" and '"'..tostring(t)..'"' or tostring(t)
      table.insert(out, tag .. val)
    end
    return table.concat(out, '\n')
  end
  return table_r(t,name or 'Value',indent or '')
end

function pr (t, name)
  writelog2(print_r(t,name))
end

function split_meta(meta)

  if meta== nil or meta=="" or meta==":" then return {'',''} end
  local a=explode(":", meta)
  if #a==1 then return {a[1],''} end
  return a
end

function set_meta(p, announcements, achievements)

  if announcements==nil then announcements='' end
  if achievements==nil then achievements='' end
  p.meta=announcements..":"..achievements
end

function implode(glue, a)

  if type(a)~="table" then return a end

  local str=""
  str=table.concat(a, glue)

  return str
end

function get_player_int_var(id, key)
  if get_player_ptr==nil then return 0; end -- maped 
  local p=get_player_ptr(id, false)
  if p==nil then return 0; end
  return vars:get_int("player."..p.name.."."..key)
end

function set_player_int_var(id, key,val)
  local p=get_player_ptr(id, false)
  if p==nil then return; end
  vars:set("player."..p.name.."."..key, val)
end

function adjacent(m, tgt)
  return m.map_level==tgt.map_level and math.abs(m.x-tgt.x)<=1 and math.abs(m.y-tgt.y)<=1
end

function copy_file(src,dst)
  
  -- fails quietly if src does not exist
  infile = io.open(src, "rb")
  if infile==nil then return end

  instr = infile:read("*all")
  infile:close()

  outfile = io.open(dst, "wb")
  outfile:write(instr)
  outfile:close()
end

