function paste_sub_map (data,lvl,hx,hy,hw,hh,mask)

  -- returns true if paste went off map

  local map=get_map_ptr(lvl)
  local idx, t
  local offmap=false
  for x=0,hw-1 do
    for y=0,hh-1 do

      idx=y*hw+x+1
      t=data[idx]
      if t~=nil and t~=mask then
        local tx=x+hx
        local ty=y+hy
        map:set_tile(tx,ty,t)
        if tx<0 or tx>=map:get_max_x() or ty<0 or ty>=map:get_max_y() then offmap=true; end
      end
    end
  end
  return offmap
end

function add_spawner(map, t, c, r, q, nearx, neary, script)

  local x=0
  local y=0
  local rr=0
  repeat
    x=nearx+math.random(0-rr,rr)
    y=neary+math.random(0-rr,rr)
    rr=rr+1
  until map:get_feat_typ(x,y)==FEATURE_NONE

  map:del_feat(x,y)
  map.spawners:add(t,c,0,map:get_level(),q,0,x,y,r,0,script)
  map:add_feat(x,y,SPAWNER); -- for maped's benefit only
end

function add_litter(map, floor, litter, chance)

  local idx, t
  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do

      if map:get_tile(x,y)==floor and math.random(1,chance)==1 then

        idx=math.random(1,#litter)
        t=litter[idx]
        map:set_tile(x,y,t)

      end
    end
  end
end

function ok_to_regen_random_lua_map(lvl)

  if server_config==nil then return true; end -- regen always in maped

  vn=string.format("last_regen_time_%03d",lvl)
  if is_offline() then
    local val=get_player_int_var(0, vn)
    if val~=0 then return false; end
    set_player_int_var(0, vn, 1)
    return true
  end

  local t=600 -- 10 min
  if lvl==1 then t=7*24*3600 ; end-- 7 days for continent

  if os.time()>vars:get_int(vn)+t then
    vars:set(vn,os.time())
    return true
  end
  return false
end

function force_map_to_regen_by_resetting_time(lvl)
  vn=string.format("last_regen_time_%03d",lvl)
  set_player_int_var(0, vn, 0)
end

function add_litter_against_walls(map, floor, wall, litter, chance, bed1, bed2)

  local idx, t
  for x=1,map:get_max_x()-2 do
    for y=1,map:get_max_y()-2 do

      if map:get_tile(x,y)==floor and (map:get_tile(x,y-1)==wall or map:get_tile(x,y+1)==wall)and math.random(1,chance)==1 then

        idx=math.random(1,#litter)
        t=litter[idx]
        map:set_tile(x,y,t)
        if t==bed1 then
          if map:get_tile(x+1,y)==floor then
            map:set_tile(x+1,y,bed2)
          else
            map:set_tile(x,y,floor)
          end
        end
      end
    end
  end
end

function spawner_nearby(map,X,Y)

  for x=X-2,X+2 do
    for y=Y-2,Y+2 do
      if map:get_spawner(x,y)~=nil then return true; end
    end
  end
  return false
end

-- returns true if route ok
function good_route(lvl,lx,ly,dx,dy,min_length)

  local map=get_map_ptr(lvl)
  local maphandler = Handler(lvl)
  local astar = AStar(maphandler)
  local path = astar:findPath(mk_node(lx,ly), mk_node(dx,dy))
  if path then
    local nodes=path:getNodes()
    if #nodes<min_length then
      map_gen_fail=true
      --writelog2("too short")
    else
      for k,v in pairs(nodes) do
        -- map:set_tile(v.location.x,v.location.y,220)
      end
      return true
    end
  else
    --writelog2("no path")
  end
  return false
end

function find_tile(map,t)

  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do
      if map:get_tile(x,y)==t then return x,y end

    end
  end
  return nil,nil

end

-- caves spawner script to set mob level - only used when caves was c++ gen. no longer used. but handy ref. see sdat/random.txt.
-- was only line in load_map_008 
function set_spawner_script(map_level, mob_level)

  local map=get_map_ptr(map_level)
  map:add_feat(1,1,SIGN,mob_level)
  for x=0,map:get_max_x()-1 do
    for y=0, map:get_max_y()-1 do

      local s=map:get_spawner(x,y)
      if s~=nil then
        s.script="level_mob_wrapper"
      end
    end
  end
end

-- this grabs it from the fea file directly
function get_map_owner(map_level, saved_player_name)

  dir="smaps"
  dot=""
  if saved_player_name~=nil then 
    dir="smaps/save"
    dot=saved_player_name.."/"
  else
    saved_player_name=""
  end
  local src=dir.."/"..dot.."lvl"..string.format("%03d", map_level)..".fea"
  infile = io.open(src, "r")
  if infile==nil then return "" end
  local count = 1
  local rc=""
  while true do
    local line = infile:read()
    if line == nil then break end
    x=explode(" ", line)
    if #x==9 and x[1]=="T" and x[2]=="1" and x[3]=="2" and x[4]=="1"  then 
      rc=x[9]
      break
    end
  end  
  infile:close()  
  return rc
end
