require 'middleclass'

island={
  13, 13, 13, 13, 13, 13, 13, 13, 13, 13,
  13, 13, 13, 126, 126, 126, 126, 126, 13, 13,
  13, 13, 126, 126, 126, 126, 126, 126, 126, 13,
  13, 126, 126, 128, 32, 32, 129, 126, 126, 13,
  126, 126, 126, 32, 66, 32, 32, 126, 126, 126,
  126, 128, 47, 59, 32, 32, 32, 129, 126, 126,
  126, 126, 131, 32, 32, 48, 32, 46, 126, 13,
  13, 126, 126, 126, 131, 32, 32, 130, 126, 13,
  13, 126, 126, 126, 126, 126, 126, 126, 126, 13,
  13, 13, 126, 126, 126, 126, 126, 126, 13, 13
}
-- problems:
-- island can go offmap
-- need to ensure can get from cave to town

-- perlin continent

function fix_perlin_height(w,h,x,y,v)

  local G=1.5 -- increase to increase fall off of the circle gradient
  local d,m

  d=math.abs(w/2-x)
  d=d-w/8
  m=(1-d/(w/2))*G
  if m<1 then v=(v+2)*m-2 end

  d=math.abs(h/2-y)
  d=d-h/8
  m=(1-d/(h/2))*G
  if m<1 then v=(v+2)*m-2 end

  --
  return v
end

function load_map_001(lvl, load_on_the_fly)

  -- if true then return end
  -- uncomment so map is randomised only on server load!
  if not ok_to_regen_random_lua_map(lvl) then return end
  local map_gen_fail=false

  math.randomseed( os.time() )

  local map=get_map_ptr(lvl)
  map:del_all_spawners()

  local w=map:get_max_x()
  local h=map:get_max_y()

  local cnt=0
  local perlin_map

  repeat
    local max=0
    local min=0
    local num_mountains=0
    perlin_map=Perlin:new(8,0.02,5,int_rand(10000))
    for x=0,map:get_max_x()-1 do
      for y=0,map:get_max_y()-1 do
        local v=fix_perlin_height(w,h,x,y,perlin_map:Get(x,y))
        if v>max then max=v; end
        if v<min then min=v; end
        if v>2 then num_mountains=num_mountains+1; end
      end
    end
    cnt=cnt+1
  until num_mountains>60 and max>2 and min<-0.8 or cnt==1000
  --if cnt==1000 then writelog2("continent mapgen fail: perlin"); end

  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do

      local v=fix_perlin_height(w,h,x,y,perlin_map:Get(x,y))

      if v<-0.8 then map:set_tile(x,y,249) -- deep water
      elseif v<-0.3 then map:set_tile(x,y,126) -- water
      elseif v<-0.1 then map:set_tile(x,y,261) -- sand - convert to sand afer placing desert towns
      elseif v>2 then map:set_tile(x,y,94) -- mountain
      elseif v>1.7 then map:set_tile(x,y,47) -- hills
      elseif v>1.2 then map:set_tile(x,y,64) -- forest
      elseif v>0.8 then map:set_tile(x,y,44) -- scrub
      else map:set_tile(x,y,32) end -- dirt

      map:del_feat(x,y)
      map:destroy_pile(x,y)
    end
  end

  -- add 2-5 swamps
  --local n=math.random(2,5)
  --for i=1,n do
  add_desert_swamp(map,43)
  --end

  --writelog2("mine")
  -- add mine somewhere
  local mx,my
  mx,my=find_mountains(map)
  map:set_tile(mx, my, 29)
  map:add_feat(mx, my, LV_CHANGE, 8)

  -- hut (player house)
  --if is_offline()  then -- and get_player_int_var(0,"hut")==1
  --  mx,my=find_forest(map)
  --  map:set_tile(mx, my, 67)
  --  map:add_feat(mx, my, LV_CHANGE, 15)
  --end

  if is_offline() and get_player_ptr~=nil then
    local p=get_player_ptr(0,false)
    map:add_feat(2,1,SIGN) -- mob level
    map:set_feat_text(2,1,p.name)
  end

  --writelog2("town")
  -- add town at least 50 paces away
  local tx, ty
  tx=0
  ty=0
  cnt=0
  repeat
    tx,ty=find_coast(map)
    cnt=cnt+1
  until cnt==100 or dist(mx,my,tx,ty)>map:get_max_x()/3 and good_route(lvl,mx,my,tx,ty,0)
  if cnt==100 then writelog2("mapgenfail: find coast"); end

  map:set_tile(tx, ty, 66)
  map:add_feat(tx, ty, LV_CHANGE, 9)

  --writelog2("island")

  local nt,st,et,wt
  local ix,iy
  nt=map:get_tile(tx,ty-1)
  st=map:get_tile(tx,ty+1)
  et=map:get_tile(tx+1,ty)
  wt=map:get_tile(tx-1,ty)

  local dx=1
  local dy=1
  if nt==126 then dx=-5; dy=-15
  elseif st==126 then dx=-5; dy=5
  elseif wt==126 then dx=-15; dy=-5
  elseif et==126 then dx=5; dy=-5
  end

  ix=tx+dx
  iy=ty+dy

  if paste_sub_map (island,1,ix,iy,10,10,13) then
    map_gen_fail=true
  end

  map:add_feat(ix+4, iy+4, LV_CHANGE, 13)
  map:add_feat(ix+5, iy+6, LV_CHANGE, 10)

  -- smooth coast
  for x=5,map:get_max_x()-5 do
    for y=5,map:get_max_y()-5 do

      t=map:get_tile(x,y)

      if t==126 then
        if not map:is_wet(x-1,y) and not map:is_wet(x-1,y-1) and not map:is_wet(x,y-1) then
          map:set_tile(x,y,130)
        end
        if not map:is_wet(x,y-1) and not map:is_wet(x+1,y-1) and not map:is_wet(x+1,y) then
          map:set_tile(x,y,131)
        end
        if not map:is_wet(x+1,y) and not map:is_wet(x+1,y+1) and not map:is_wet(x,y+1) then
          map:set_tile(x,y,128)
        end
        if not map:is_wet(x,y+1) and not map:is_wet(x-1,y+1) and not map:is_wet(x-1,y) then
          map:set_tile(x,y,129)
        end
      end
    end
  end

  -- build an a star to test if path exists from cave to town
  map_gen_fail=not good_route(lvl,mx,my,tx,ty,0)

  -- fix temp sand
  a={45}
  add_litter(map,261,a,1)
  a={196} -- rand peaks
  add_litter(map,94,a,15)

  -- turn dirt into grass and stuff
  a={46,56,56,58,59}
  add_litter(map,32,a,5)

  -- global spanwers
  --add_spawner(map, 1, 0xd, 0, 15, 1, 1)
  --add_spawner(map, 1, 0xc, 0, 15, 1, 2)
  add_spawner(map, 0, 0x207, 0, 40, 1, 3, "level_mob_wrapper")
  add_spawner(map, 0, 0x206, 0, 10, 1, 4, "level_mob_wrapper")
  add_spawner(map, 0, 0x208, 0, 10, 1, 5, "level_mob_wrapper")
  add_spawner(map, 0, 0x203, 0, 10, 1, 6, "level_mob_wrapper")

  map:add_feat(1,1,SIGN,11) -- mob level

  -- make sure only water or sand is on the map edges
  for x=0,map:get_max_x()-1 do
    if not good_edge_tile(map,x,0) then map_gen_fail=true; end
    if not good_edge_tile(map,x,map:get_max_y()-1) then map_gen_fail=true; end
  end
  for y=1,map:get_max_y()-2 do
    if not good_edge_tile(map,0,y) then map_gen_fail=true; end
    if not good_edge_tile(map,map:get_max_x()-1,y) then map_gen_fail=true; end
  end

  if map_gen_fail then
    if vars~=nil then force_map_to_regen_by_resetting_time(lvl) end -- force to regen
    load_map_001(lvl, load_on_the_fly)
  else
    map:save_map()
    map:save_features()
    map:save_items()
  end
end

function find_mountains(map)

  --writelog2("find mountains")
  local x,y
  local cnt=0
  repeat

    x=math.random(10,map:get_max_x()-10)
    y=math.random(10,map:get_max_y()-10)
    nt=map:get_tile(x,y-1)
    st=map:get_tile(x,y+1)
    et=map:get_tile(x+1,y)
    wt=map:get_tile(x-1,y)
    cnt=cnt+1
  until cnt==1000 or map:get_tile(x,y)==94 and (nt==47 or st==47 or et==47 or wt==47) and is_remote(map,x,y)
  --if cnt==1000 then writelog2("mapgenfail: find mountains"); end

  return x,y
end

function find_coast(map)

  --writelog2("find coast")
  local x,y
  local cnt=0
  repeat

    x=math.random(10,map:get_max_x()-10)
    y=math.random(10,map:get_max_y()-10)
    nt=map:get_tile(x,y-1)
    st=map:get_tile(x,y+1)
    et=map:get_tile(x+1,y)
    wt=map:get_tile(x-1,y)
    cnt=cnt+1
  until cnt==1000 or map:get_tile(x,y)==261 and (nt==126 and is_remote_t(map,x,y+3,32,1) or st==126 and is_remote_t(map,x,y-3,32,1) or et==126 and is_remote_t(map,x-3,y,32,1) or wt==126 and is_remote_t(map,x+3,y,32,1))
  --if cnt==1000 then writelog2("mapgenfail: find coast (func)"); end

  return x,y
end

function is_remote(map,x,y)

  for j=y-9,y+9 do
    for i=x-9,x+9 do
      if map:get_feat_typ(i,j)~=FEATURE_NONE then return false; end
    end
  end
  return true
end

function is_remote_t(map,x,y,t,r)

  for j=y-r,y+r do
    for i=x-r,x+r do
      if map:get_tile(i,j)~=t then return false; end
    end
  end
  return true
end

function add_desert_swamp(map,t)

  --wristelog2("add desert swamp")
  local cnt=0
  local x=0
  local y=0
  repeat

    x=math.random(10,map:get_max_x()-10)
    y=math.random(10,map:get_max_y()-10)
    cnt=cnt+1
  until cnt==1000 or map:get_tile(x,y)==32 and is_remote_t(map,x,y,32,5)
  --if cnt==1000 then writelog2("mapgen fail: add desert swamp"); end

  local r=math.random(7,15)
  for i=x-r,x+r do
    for j=y-r,y+r do
      if math.sqrt((x-i)*(x-i)+(y-j)*(y-j))<r+math.random(-2,2) and map:get_tile(i,j)==32 then map:set_tile(i,j,t); end
    end
  end

  return x,y
end

function mk_node(x,y)

  local node={}
  node.x=x
  node.y=y
  return node
end

function good_edge_tile(map,x,y)

  local t=map:get_tile(x,y)
  return t==249 or t==126 or t==45 or t>=128 and t<=131
end

