function load_map_013(lvl, load_on_the_fly)

  -- uncomment so map is randomised only on server load!
  if not ok_to_regen_random_lua_map(lvl) then return end

  local map=get_map_ptr(lvl)
  map:del_all_spawners()

  local roomfloor=30
  local outsidefloor=32

  fd_random_dungeon(map, roomfloor, 200, 0, 201, 202, 203, outsidefloor, 1)

  local l
  l={204,205}
  add_litter(map, 200, l, 10)

  l={44,51,46,52,55,56,57,58,59,45}
  add_litter(map, 32, l, 10)

  l={31,51}
  add_litter(map,30,l,30)

  add_litter(map,30,{74},300) -- lockers

  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do
      map:del_feat(x,y)
      map:destroy_pile(x, y)
    end
  end

  for x = 2, map:get_max_x() - 3, 5 do
    for y = 2, map:get_max_y() - 3, 5 do
      if distance(x,y,math.floor(map:get_max_x()/2),map:get_max_y()-3)<15 then
        -- nothing: too close to entrance
      elseif math.random(1,5) ==1 and (map:get_tile(x,y) == roomfloor or map:get_tile(x,y) == outsidefloor) then -- spawn some mobs in the rooms
        map:del_feat(x,y)
        add_spawner(map, 0, 0x209, 3, 3, x, y, "level_mob_wrapper")
      elseif math.random(1,60) == 1 and (map:get_tile(x,y) == roomfloor or map:get_tile(x,y) == outsidefloor) then -- spawn some chests
        add_spawner(map, 2, math.random(5,10), -1, 1, x, y, "")
      end
    end
  end

  map:add_feat(1,1,SIGN,15) -- mob level

  map:save_map()
  map:save_features()
  map:save_items()

end

-- by rusty patterson of EUO, modified for EAO by MB

function fd_random_dungeon(map, roomfloor, roomwall, wallsecretdoor, wallwindow, brokenwindow, door, outsidefloor, finallevel)

  mw = map:get_max_x()
  mh = map:get_max_y()
  midx=math.floor(mw/2)

  -- add floor for rooms and area outside rooms, remove any preexisting features or items, spawn mobs and chests
  for x = 0, mw-1, 5 do
    for y = 0, mh-1, 5 do
      t = math.random(1,3)

      if (y<mh-20 or x<midx-20 or x>midx+20) and t == 1 then
        for x2 = 0, 4 do
          for y2 = 0, 4 do
            map:set_tile(x + x2, y + y2, roomfloor)
            map:del_feat(x + x2, y + y2)
            map:remove_item(x + x2, y + y2)
          end
        end
      else
        for x2 = 0, 4 do
          for y2 = 0, 4 do
            map:set_tile(x + x2, y + y2, outsidefloor)
            map:del_feat(x + x2, y + y2)
            map:remove_item(x + x2, y + y2)
          end
        end
      end
    end
  end

  -- if finallevel = 1 then make large room for boss
  if finallevel == 1 then
    rw = 20
    rh = 20
    rx = mw / 2 - rw / 2
    ry = 10
    for x = rx, rx + rw - 1 do
      for y = ry, ry + rh - 1 do
        if x == rx or x == rx + rw - 1 or y == ry or y == ry + rh - 1 then
          map:set_tile(x,y,roomwall)
        else
          map:set_tile(x,y,20)
        end
      end
    end
    bx=mw/2
    by=rh/2+10
    add_spawner(map, 0, 0x207, -1, 1, bx, by, "final_boss")
  end

  -- remove the touching corners
  for x = 0, mw-2 do
    for y = 0, mh-2 do
      if map:get_tile(x,y) == map:get_tile(x+1,y+1) and map:get_tile(x+1,y) == map:get_tile(x,y+1) then
        if math.random(1,2) == 1 then
          t = map:get_tile(x,y)
        else
          t = map:get_tile(x+1,y)
        end
        map:set_tile(x,y,t)
        map:set_tile(x+1,y,t)
        map:set_tile(x,y+1,t)
        map:set_tile(x+1,y+1,t)
      end
    end
  end

  -- add walls around rooms
  for x = 0, mw-1 do
    for y = 0, mh-1 do
      t = map:get_tile(x,y)
      if t == outsidefloor then
        if x > 0 and map:get_tile(x-1,y) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif x < mw-1 and map:get_tile(x+1,y) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif y > 0 and map:get_tile(x,y-1) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif y < mh-1 and map:get_tile(x,y+1) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif x > 0 and y > 0 and map:get_tile(x-1,y-1) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif x < mw-1 and y < mh-1 and map:get_tile(x+1,y+1) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif y > 0 and x < mw-1 and map:get_tile(x+1,y-1) == roomfloor then
          map:set_tile(x,y,roomwall)
        elseif y < mh-1 and x > 0 and map:get_tile(x-1,y+1) == roomfloor then
          map:set_tile(x,y,roomwall)
        end
      end

    end
  end

  -- add walls around rooms
  for x = 0, mw-1 do
    for y = 0, mh-1 do
      t = map:get_tile(x,y)
      if x == 0 or y == 0 or x == mw-1 or y == mh-1 then
        if t==32 then map:set_tile(x,y,84)
        else map:set_tile(x,y,200) end
      end
    end
  end

  -- add doors and windows, doors are placed where all areas will be accessable.
  for x = 2, mw-3, 5 do
    md = math.random(0,1)
    for y = 1, mh-2 do
      if map:get_tile(x,y) == roomwall then
        if md == 1 then
          md = 0
          rd = math.random(1,3)
          if rd == 1 then
            map:set_tile(x,y,door) -- normal door
          elseif rd == 2 then
            map:set_tile(x,y,door) -- windowed door
          elseif rd == 3 and wallsecretdoor~=0 then
            map:set_tile(x,y,wallsecretdoor) -- secret door
          end
          tl = math.random(1,10) -- 20% chance the door is trapped or 20% chance the door is locked
          if tl == 1 then
            map:add_feat(x,y,TRIG_TRAP,0,10,5)
          elseif tl == 2 then
            map:add_feat(x,y,TRIG_LOCK)
          end
        else
          md = 1
          if math.random(1,3) == 1 then
            if int_rand(2)==0 then
              map:set_tile(x,y,wallwindow) -- window
            else
              map:set_tile(x,y,brokenwindow) -- window
            end
          end
        end
      end
    end
  end
  for y = 2, mh-3, 5 do
    md = math.random(0,1)
    for x = 1, mw-2 do
      if map:get_tile(x,y) == roomwall then
        if md == 1 then
          md = 0
          rd = math.random(1,3)
          if rd == 1 then
            map:set_tile(x,y,door) -- normal door
          elseif rd == 2 then
            map:set_tile(x,y,door) -- windowed door
          elseif rd == 3 and wallsecretdoor~=0 then
            map:set_tile(x,y,wallsecretdoor) -- secret door
          end
          tl = math.random(1,10) -- 20% chance the door is trapped or 20% chance the door is locked
          if tl == 1 then
            map:add_feat(x,y,TRIG_TRAP,0,10,5)
          elseif tl == 2 then
            map:add_feat(x,y,TRIG_LOCK)
          end
        else
          md = 1
          if math.random(1,3) == 1 then
            if int_rand(2)==0 then
              map:set_tile(x,y,wallwindow) -- window
            else
              map:set_tile(x,y,brokenwindow) -- window
            end
          end
        end
      end
    end
  end

  map:set_tile(0,0,32)

  for x=midx-5,midx+5 do
    for y=mh-5,mh-1 do
      map:set_tile(x,y,32)
    end
  end

end

function final_boss(m)

  if is_offline() and get_player_int_var(0,"rustavinious_dead")==1 then
    m:set_name("just a cow ...")
  else
    m:set_align(ALIGN_EVIL)
    set_monster_level(m,22)
    m:mk_boss()
    m.hide_boss_name=false
    m:set_name("Rustavinious")
    m.large=true
    m:set_hp_max(1000)
    m.hp=1000
    m.gives_ear=true

    m:set_intel(100)
    m:set_mana_max(100)

    m:learn_spell("lightningauto")
    m:learn_spell("ip2")
    m.morale=0
    m.never_retreat=true
    m.immune_to_glass_bleed_hex=true

    m.death_func="open_portal_to_end"
    m.heartbeat_func="rustavinious_actions"
  end
end

function open_portal_to_end(m)

  local x=50
  local y=18

  local map=get_map_ptr(m.map_level)
  map:del_feat(x,y)
  send_event(m.map_level,m.id,x,y,EVENT_PORTAL_UP,0);
  map:add_feat(x,y,TRIG_PORTAL,14,1,-1,Item(-1))

  if is_offline() then 
    set_player_int_var(0,"rustavinious_dead",1) 
  end
end
