function make_vault(lvl, load_on_the_fly, up, down, mob_level, room_wall, window, door, secret)

  -- uncomment so map is randomised only on server load!
  if not ok_to_regen_random_lua_map(lvl) then return end
  local map_gen_fail=false

  local map=get_map_ptr(lvl)
  map:set_size(100,100,true)
  map:del_all_spawners()

  -- reset to all walls
  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do

      t=87 -- default wall
      map:set_tile(x, y, t)
      map:del_feat(x, y)
      map:destroy_pile(x, y)

    end
  end

  -- map monster lvl goes into 1,1
  map:add_feat(1,1,SIGN,mob_level)

  -- chest spawner
  map.spawners:add(2,mob_level,0,lvl,math.random(5,10),0,2,2,0)
  map:add_feat(2,2,SPAWNER); -- for maped's benefit only

  -- rooms
  rooms_across=5
  rooms_down=5
  rooms=rooms_across*rooms_down

  sx=10
  sy=3
  centres={}
  bigRoom=math.floor(rooms_down/2)*rooms_across+math.floor(rooms_across/2)+1
  local lastVault=(lvl==4)
  for r=1,rooms do

    sy=math.floor((r-1)/rooms_across)*18+2-int_rand(5)+10
    if sy<1 then sy=1 end

    ry=math.floor((r-1)/rooms_across)
    rx=(r-1)%rooms_across

    w=int_rand(6)+5
    h=int_rand(6)+5

    if lastVault and r==bigRoom then
      h=21
      w=21
    end

    for x=sx,sx+w+1 do
      for y=sy,sy+h+1 do

        floor=20
        if lastVault and r==bigRoom then floor=19 end
        if x>sx and x<sx+w+1 and y>sy and y<sy+h+1 then map:set_tile(x,y,floor)
        else map:set_tile(x,y,room_wall)
        end
      end
    end

    c={sx+math.floor(w/2), sy+math.floor(h/2)}
    table.insert(centres,c)

    sx=sx+w+5+int_rand(4)

    if r%rooms_across==0 then
      sx=10 +int_rand(5) -2
    end

  end

  -- builds a 2d array of adjacent rooms for each room
  adjRooms={}
  for i=1,rooms do
    a={}
    x=(i-1)%rooms_across+1
    y=1+math.floor((i-1)/rooms_across)

    if y>1 then table.insert(a, i-rooms_across) end
    if y<rooms_down then table.insert(a, i+rooms_across) end
    if x>1 then table.insert(a, i-1) end
    if x<rooms_across then table.insert(a, i+1) end
    table.insert(adjRooms, a)
  end

  -- using connection algorithm from rogue
  -- http://kuoi.com/~kamikaze/GameDesign/art07_rogue_dungeon.php
  -- ladders go at first and last connected rooms

  connected={}
  for i=1,rooms do
    connected[i]=false
  end

  -- first pass - basically draws a linear start to finish
  r=1+int_rand(rooms)
  start_room=r
  connected[r]=true

  while true do

    cnt=0
    repeat
      r2=get_random_element(adjRooms[r])
      cnt=cnt+1
    until not connected[r2] or cnt==100

    if cnt==100 then break end -- this is weak but I am in a hurry

    draw_corridor(map, centres[r][1], centres[r][2], centres[r2][1], centres[r2][2], room_wall)

    connected[r2]=true
    r=r2
    end_room=r

  end

  -- writelog2("sr="..start_room.." er="..end_room);

  -- 2nd pass - keep trying to connect each unconnected room until there are no unconnected rooms left
  -- actually just do this n times and leave some unconnected

  cnt=0
  repeat

    for r=1,rooms do

      if not connected[r] then
        n=get_random_element(adjRooms[r])
        if connected[n] then
          draw_corridor(map, centres[r][1], centres[r][2], centres[n][1], centres[n][2], room_wall)
          connected[r]=true
        end
      end

    end

    cnt=cnt+1

  until cnt==100

  -- doors
  for x=0,map:get_max_x()-1 do
    for y=0,map:get_max_y()-1 do

      if map:get_tile(x,y)==19 and map:get_tile(x+1,y)==room_wall and map:get_tile(x-1,y)==room_wall and (map:get_tile(x,y-1)==20 or map:get_tile(x,y+1)==20) then
        map:set_tile(x, y, door)
      end
      if map:get_tile(x,y)==19 and map:get_tile(x,y+1)==room_wall and map:get_tile(x,y-1)==room_wall and (map:get_tile(x-1,y)==20 or map:get_tile(x+1,y)==20) then
        map:set_tile(x, y, door)
      end

    end
  end

  -- decorations
  a={70,71,72,73,74,160}
  add_litter_against_walls(map, 20, room_wall, a, 4, 160, 161)

  -- spawners and lights
  add_spawners_and_lights(map,lvl,centres)

  -- ladder up
  d=start_room
  if lastVault then d=bigRoom end

  lx=centres[d][1]
  ly=centres[d][2]
  map:set_tile(lx, ly, 62)
  map:del_feat(lx, ly)
  map:destroy_pile(lx, ly)
  map:add_feat(lx, ly, LV_CHANGE, up)
  --writelog2("d="..d.." lx="..lx.." ly="..ly)

  -- down
  dx=centres[end_room][1]
  dy=centres[end_room][2]
  map:set_tile(dx, dy, 60)
  map:del_feat(dx, dy)
  map:destroy_pile(dx, dy)
  map:add_feat(dx, dy, LV_CHANGE, down)
  --writelog2("u="..end_room.." lx="..lx.." ly="..ly)

  -- dog
  if lastVault then
    x=centres[bigRoom][1]-5
    y=centres[bigRoom][2]-4
    t=17
    map:set_tile(x-1, y-1, t)
    map:set_tile(x, y-1, t)
    map:set_tile(x+1, y-1, t)
    map:set_tile(x-1, y, t)
    map:set_tile(x+1, y, t)
    map:set_tile(x-1, y+1, t)
    map:set_tile(x+1, y+1, t)

    map:add_item(x-1,y+2,1,Item(0x51b))
    map:add_feat(x,y+1,TRIG_BARRIER,1)
    map:add_feat(x-1,y+2,TRIG_SWITCH,0,0,0,Item(0),"rescue_dog")

    if not is_maped() and (is_online() or get_player_int_var(0, "has_bathplug")==0) then
      add_spawner(map, 0, 0x201, -1, 1, x, y, "bathplug_init")
    end

  end

  map_gen_fail=not good_route(lvl,lx,ly,dx,dy,150)
  maybe_add_vending_machine(map,lvl)

  if map_gen_fail then
    if vars~=nil then force_map_to_regen_by_resetting_time(lvl) end -- force to regen
    make_vault(lvl, load_on_the_fly, up, down, mob_level, room_wall, window, door, secret)
  else
    map:save_map()
    map:save_features()
    map:save_items()
  end

end

function draw_corridor(map, x1, y1, x2, y2, room_wall)

  dx=0
  dy=0

  northSouth=false
  toDoMidpoint=0

  if math.abs(x1-x2) < math.abs(y1-y2) then
    northSouth=true

    start=0
    thickness=0
    step=1
    if y1>y2 then step=-1 end
    for y=y1,y2,step do

      if start==0 and map:get_tile(x1,y)==87 then
        start=y
      end
      if start>0 and map:get_tile(x2,y)==87 then thickness=thickness+1 end
    end

    toDoMidpoint=math.floor(thickness/2)+math.abs(start-y1)
    -- writelog2("x1="..x1.." y1="..y1.." thickness="..thickness.." step="..step.." start="..start.." mp="..toDoMidpoint)
  else

    start=0
    thickness=0
    step=1
    if x1>x2 then step=-1 end
    for x=x1,x2,step do

      if start==0 and map:get_tile(x,y1)==87 then
        start=x
      end
      if start>0 and map:get_tile(x,y2)==87 then thickness=thickness+1 end
    end

    toDoMidpoint=math.floor(thickness/2)+math.abs(start-x1)
    -- writelog2("x1="..x1.." y1="..y1.." thickness="..thickness.." step="..step.." start="..start.." mp="..toDoMidpoint)
  end

  -- if northSouth then return end

  if x2<x1 then dx=-1
  elseif x2>x1 then dx=1
  end

  if y2<y1 then dy=-1
  elseif y2>y1 then dy=1
  end

  steps=math.abs(x1-x2)+math.abs(y1-y2)-1

  x=x1
  y=y1
  cnt=0
  chkMidpoint=false
  doingHalfWay=false
  while cnt<steps do

    -- if half way lets move to do the smaller bit
    if not chkMidpoint then
      if cnt==toDoMidpoint then
        doingHalfWay=true
        chkMidpoint=true
      end
    end

    if not doingHalfWay and northSouth or not northSouth and doingHalfWay then
      y=y+dy
    elseif not doingHalfWay and not northSouth or northSouth and doingHalfWay then
      x=x+dx
    end

    if map:get_tile(x,y)==87 or map:get_tile(x,y)==room_wall then map:set_tile(x,y,19) end

    if doingHalfWay then
      if not northSouth and y==y2 or northSouth and x==x2 then
        doingHalfWay=false
      end
    end

    cnt=cnt+1
  end
end

