BCODE_GOLD=0x100

spells_in_tome_drops={
  0x1,0x19,0x0,0x18,0x3a,0x10,0x56, -- mage tomes here
  0x2e,0x2f,0x35,0x63,0x2d -- fighter tomes
}

function get_ammo()

  local ammo={0x150,0x151,0x152,0x153}
  return Item(get_random_element(ammo)), math.random(10,20)
end

function get_rand_provision(quality,id)

  if(quality==nil) then quality=0; end
  if(id==nil) then id=-1; end
  -- returns code, qty

  -- engineers and droids more likely to get spare parts
  local p=get_player_ptr(id)
  if p~=nil and (sc:get_classname(p.cls)=="engineer" or sc:get_racename(p.race)=="droid") and int_rand(3)==1 then
    return Item(0x147), math.random(3,6) -- spareparts
  end

  local r=math.random(1, 6)

  if(r==1) then
    return Item(0x141), 1 -- probe
  elseif(r==2) then
    return Item(0x11c), math.random(1,3) -- lockpick
  elseif(r==3) then
    return Item(0x521), 1 -- skullkey
  elseif(r==4) then
    return Item(0x177), math.random(7,12) -- bandage
  end

  -- default
  return get_ammo()
end

function get_rand_apothecary_item(quality, id)

  local colour=math.random(0,6)
  if is_online() or knows_pill_colour(colour) then
    return get_known_pill_item_from_colour(colour), 1
  else
    return make_unknown_pill_item(colour), 1
  end
end

function get_rand_food()

  local foodArray={0x160,0x161,0x162,0x163,0x164,0x165,0x166}
  local itm=get_random_element(foodArray)
  local qty=math.random(1,3)

  return Item(itm), qty -- food
end

function chest_loot (quality,x,y,z)

  local item_cnt=xdy(3,quality/3)
  if item_cnt<1 then item_cnt=1 end
  for j=1,item_cnt do

    local itm=0
    local quantity=1

    -- gaps left intentionally to create empty chests
    local r=math.random(0,100)

    if r>=70 then

      itm=wl:get_rand_weapon_armour(quality)
      if int_rand(2)==1 then
        itm=random_enchant(itm,quality)
      else
        itm=random_material(itm)
      end

      -- prov & tools
    elseif(r>60 and r < 70) then
      itm,quantity=get_rand_provision(quality)

      -- food
    elseif(r>50 and r < 60) then

      itm,quantity=get_rand_food()

      -- potion or scroll
    elseif true or (r>25 and r < 50) then

      itm,quantity=get_rand_apothecary_item(quality)

    elseif r<25 then

      itm=Item(BCODE_GOLD)
      quantity=math.random(0,15*quality)+5*quality
    end

    if itm~=0 then
      map_add_item(x, y, z, itm, quantity)
      local ammo=wl:get_ammo_type(itm)
      if ammo~=0 then
        map_add_item(x, y, z, ammo, math.random(15,25))
      end
    end
  end
end

function mob_loot(mon, attacker_id, owner)

  local global_drop_rate_decrease=1
  local x=mon.x
  local y=mon.y
  local treasure = mon.treasure

  local other = bitwiseAnd(treasure , 0xfff00000) / 0x100000
  local quality = bitwiseAnd(treasure , 0x000f0000) / 0x10000
  local chc_art = bitwiseAnd(treasure , 0x0000f000) / 0x1000
  local chc_apoth = bitwiseAnd(treasure , 0x00000f00) / 0x100
  local chc_wep = bitwiseAnd(treasure , 0x000000f0) / 0x10
  local chc_gold = bitwiseAnd(treasure , 0x0000000f)

  local smap=get_map_ptr(mon.map_level);

  -- check for hook, bail if so
  if(smap:feature_is_hook(x,y)) then return; end

  local add_chest=false
  if mon.boss and chc_wep>0 and int_rand(2)==0 then
    add_chest=true
  end

  local itm=0, q
  if int_rand(16*global_drop_rate_decrease) < chc_apoth then

    -- drop potion or scroll
    local r=int_rand(100)
    local qty=1
    local food=false
    if(r<40) then

      itm,qty=get_rand_apothecary_item(quality, attacker_id)

    elseif(r<80) then

      itm,qty=get_rand_provision(quality, attacker_id)
    else

      itm,qty=get_rand_food()
      food=true
    end

    if(itm~=0) then
      if add_chest then smap:add_item(x, y, qty, itm, true, game_time, owner);
      else smap:add_item_on_empty_tile(x, y, qty, itm, true, game_time, owner);
      end
    end
  end

  -- gold
  if(int_rand(32) < chc_gold) then

    -- gold isn't dropped anymore, its given to the killer or killing party
    -- if monster was killed not by a player, its dropped on smap
    q=int_rand(10*quality);

    if(attacker_id<100) then

      divide_loot(attacker_id, Item(BCODE_GOLD), q);
    else
      if add_chest then smap:add_item(x, y, q, Item(BCODE_GOLD), true, game_time, owner);
      else smap:add_item_on_empty_tile(x, y, q, Item(BCODE_GOLD), true, game_time, owner);
      end
    end

  end

  -- if monster is not boss and carrying weapon
  if not mon.boss and mon.weapon~=0 then
    if int_rand(100)<30 then
      smap:add_item_on_empty_tile(x, y, 1, mon.weapon, true, game_time, owner)
    end
    local ammo=wl:get_ammo_type(mon.weapon)
    if ammo~=0 then
      local aqty=math.random(5,10)
      smap:add_item_on_empty_tile(x, y, aqty, ammo, true, game_time, owner)
    end
  end

  -- artifact drop
  local may_add_chest=false
  local rc,arty=artifact_drop_check(chc_art)
  -- if(mon.boss and chc_art>0 and int_rand((16-chc_art)^1.65 + 14)==1 and int_rand(30)==1) then
  if mon.boss and rc then

    itm=arty
    if itm:isValid() then -- redundant
      -- tint it - dont forget that rand material has a medium chc of tinting, not guaranteed
      if get_material(itm)==0 then
        itm=random_material(itm)
      end

      if add_chest then smap:add_item(x, y, 1, itm, true, game_time, owner)
      else smap:add_item_on_empty_tile(x, y, 1, itm, true, game_time, owner)
      end
    end

    -- weapon drops (if not an artifact)
  elseif (int_rand(16*global_drop_rate_decrease) < chc_wep or mon.boss and chc_wep>0) then

    -- uncomment when add jewellery
    --if(int_rand(6)==1 and mon.boss) then itm=wl:get_rand_jewellry() -- one chance in 6 of dropping jewellry
    --else
    itm=wl:get_rand_weapon_armour(quality)
    --end

    if itm:isValid() then

      if mon.boss then
        q=quality
        if int_rand(100)<25 then q=q+1 end -- used to me +/- 1 rand but 0xf qual bosses give disappointing loot often
        itm=random_enchant(itm, q)
      end

      -- only bosses drop
      if mon.boss then -- was get_base_code(itm)~=itm

        if add_chest then smap:add_item(x, y, 1, itm, true, game_time, owner)
        else smap:add_item_on_empty_tile(x, y, 1, itm, true, game_time, owner)
        end
        local ammo=wl:get_ammo_type(itm)

        -- add a tome maybe
        if attacker_id<100 then

          -- tome drops
          local p=get_player_ptr(attacker_id)
          for i,spell_id in ipairs(spells_in_tome_drops) do
            local tome=Item(0x0000050f + 0x100000*spell_id)
            local bankedTome=Item(0x4000050f + 0x100000*spell_id)
            local sk=spells:get_skill(spell_id)
            local skn=sc:get_skillname(sk)
            if spells:class_can_use(spell_id, p.cls) and not p:has_spell(spell_id) and get_player_item_qty(p.id, tome)==0 and get_player_item_qty(p.id, bankedTome)==0 and (skn=="evangelism" or int_rand(3)==0) then
              if add_chest then smap:add_item(x, y, 1, tome, true, game_time, owner);
              else smap:add_item_on_empty_tile(x, y, 1, tome, true, game_time, owner);
              end
              break
            end
          end
        end
        -- ammo
        if ammo~=0 then
          local aqty=math.random(15,30)
          if add_chest then smap:add_item(x, y, aqty, ammo, true, game_time, owner);
          else smap:add_item_on_empty_tile(x, y, aqty, ammo, true, game_time, owner);
          end
        end
      end
    end
  end

  -- add a chest on top
  -- disabled due to items being 'spread' - was kinda an annoyance anyway
  if add_chest then

    local locked=0
    local trapped=0
    if int_rand(7)==1 then locked=2 end
    -- if int_rand(4)==1 and not PERMADEATH then trapped=4 end

    local chest=make_code(0x108, 0, quality, 1 + locked + trapped )
    smap:add_item(x, y, 1, chest, true) -- note that no game_time is set here - so the chest doesn't decay
  end

end

function artifact_drop_check(rarity)
  return false, 0
end
